﻿--[[
	LootMaster
    
			FMI:
				http://www.wowwiki.com/ItemEquipLoc	Different item slots where item can be placed

			Pass on loot == (GetOptOutOfLoot() == true)
]]

LootMaster          = LibStub("AceAddon-3.0"):NewAddon("EPGPLootMaster", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

local version 	    = "v0.3.24"
local dVersion 	    = "2009-06-28T02:45:54Z"
local iVersion	    = 3
local iVersionML	= 11
local _G            = _G

local debug         = false
local addon         = LootMaster		-- Local instance of the addon

local HandleChatEvents = {
    ['CHAT_MSG_WHISPER_INFORM']     = true, -- Filter outgoing whispers.
    ['CHAT_MSG_PARTY']              = true,
    ['CHAT_MSG_RAID']               = true,
    ['CHAT_MSG_RAID_LEADER']        = true
}

--[[
    Returns a table serialized as a string.
]]
function tprint( data, level )
	level = level or 0
	local ident=strrep('    ', level)
	if level>5 then return end

	if type(data)~='table' then print(tostring(data)) end;

	for index,value in pairs(data) do repeat
		if type(value)~='table' then
			print( ident .. '['..index..'] = ' .. tostring(value) .. ' (' .. type(value) .. ')' );
			break;
		end
		print( ident .. '['..index..'] = {')
        tprint(value, level+1)
        print( ident .. '}' );
	until true end
end

function LootMaster:GetVersionString()
    return self.version;
end
    
function LootMaster:OnInitialize()
    
    self.version 	= version;
    self.dVersion 	= dVersion;
    self.iVersion 	= iVersion;
    
    self.db = LibStub("AceDB-3.0"):New("EPGPLootMaster")
    
    self.db:RegisterDefaults(
    {
      profile = {
        auto_announce_threshold = 4,
        AutoLootThreshold = 2,
        hideOnSelection = true,
        loot_timeout = 60,
        filterEPGPLootmasterMessages = true,
        monitor = false,
        monitorSend = true,
        monitorThreshold = 2,
        ignoreResponseCorrections = false,
        hideMLOnCombat = true,
        hideSelectionOnCombat = false
      }
    })
    
    self.lootList   = {};
    self.lootMLCache = {};
    
    -- Client responses    
    self.RESPONSE = {
        { ["CODE"]      = "NOTANNOUNCED",   ["COLOR"] = {1,1,1},        ["TEXT"] = 'Not yet announced to candidate' },
        { ["CODE"]      = "INIT",           ["COLOR"] = {1,0,0},        ["TEXT"] = 'Offline or lootmaster not installed?' },
        { ["CODE"]      = "WAIT",           ["COLOR"] = {1,0.5,0},      ["TEXT"] = 'Making selection, please wait...' },
        { ["CODE"]      = "TIMEOUT",        ["COLOR"] = {1,0,1},        ["TEXT"] = 'Candidate did not respond on time.' },
        { ["CODE"]      = "NEED",           ["COLOR"] = {0.4,1,0.4},    ["TEXT"] = 'Need / Mainspec' },
        { ["CODE"]      = "GREED",          ["COLOR"] = {1,1,0},        ["TEXT"] = 'Greed / Offspec' },
        { ["CODE"]      = "DISENCHANT",     ["COLOR"] = {0,0.8,1},      ["TEXT"] = '--disenchanter--' },
        { ["CODE"]      = "PASS",           ["COLOR"] = {0.6,0.6,0.6},  ["TEXT"] = 'Pass' }
    }    
    for i,d in ipairs(self.RESPONSE) do
        self.RESPONSE[d.CODE] = i;
        self.RESPONSE[d.CODE .. "_TEXT"] = d.TEXT;
    end    
    LootMaster.RESPONSE = self.RESPONSE;
    
    -- Loot receive types.
    self.LOOTTYPE = {
        { ["CODE"]      = "UNKNOWN",        ["TEXT"] = '%s received %s for unknown reason%4$s.' },
        { ["CODE"]      = "GP",             ["TEXT"] = '%s received %s for %s GP%s.' },
        { ["CODE"]      = "DISENCHANT",     ["TEXT"] = '%s received %s for disenchantment%4$s.' },
        { ["CODE"]      = "BANK",           ["TEXT"] = '%s received %s for bank%4$s.' }
    }    
    for i,d in ipairs(self.LOOTTYPE) do
        self.LOOTTYPE[d.CODE] = i;
        self.LOOTTYPE[d.CODE .. "_TEXT"] = d.TEXT;
    end    
    LootMaster.LOOTTYPE = self.LOOTTYPE;
    
	-- Register communications
	self:RegisterComm("EPGPLootMasterC",    "CommandReceived")
    
    -- Register communications for version checking
	self:RegisterComm("EPGPLMVChk", 	    "CommVersionCheckRequest")
	self:RegisterComm("EPGPLMVRsp",	        "CommVersionCheckResponse")
	self:RegisterComm("EPGPLMVHdlr",	    "CommVersionCheckHandler")
    
    -- Check for updates versions in the guild
	self:SendCommMessage("EPGPLMVChk",      iVersion .. "_" .. version, "GUILD")
    
    self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat");
    self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat");

	self:RegisterChatCommand("lm",	        "SlashHandler")	
    
    self:RegisterChatCommand("rl", function() ReloadUI() end)   
        
    self:Print(format('%s loaded.', version))    
end

function LootMaster:OnEnable()
    -- Postpone the chathooks to make sure we're the last hooking these.
    self:ScheduleTimer("PostEnable", 1)
end

function LootMaster:PostEnable()    
    -- Inbound Chat Hooking
    self:RawHook("ChatFrame_MessageEventHandler", true)
end

local lastMsgID = nil;
local lastMsgHandled = true;

--[[
    Filter out 'EPGPLootmaster:' announces in the raid and raid leader and party chat
]]--
function LootMaster:ChatFrame_MessageEventHandler(this, event, message, ...)
 
    if LootMaster.db.profile.filterEPGPLootmasterMessages and HandleChatEvents[event] then
        
        -- Lets speed this up by checking if we already tested the message
        local msgID = select(select("#", ...), ...)
        if lastMsgID == msgID then
            if not lastMsgHandled then
                return false;
            end
        else
            lastMsgID       = msgID;
            lastMsgHandled  = true;
            
            -- find EPGPLootmaster: in the chat message and prevent these messages from showing up.
            if strfind(message, '^%s*EPGPLootmaster:%s+') then  
                lastMsgHandled  = false;
                return false;
            end       
        end        
    end
    
    return self.hooks["ChatFrame_MessageEventHandler"](this, event, message, ...) 
end

function LootMaster:SlashHandler( input )
	local _,_,command, args = string.find( input, "^(%a-) (.*)$" )
	command = command or input

	if command=='version' or command=='versioncheck' or command=='vc' or command=='v' then
        
        self:ShowVersionCheckFrame();
        
	elseif command=='debug' then
        
        self.debug = not self.debug;
        if self.debug then
            self:Print('Debugging enabled')
        else
            self:Print('Debugging disabled')
        end
        
    elseif command=='config' or command=='c' or command=='options' or command=='o' then
        
        InterfaceOptionsFrame_OpenToCategory("EPGPLootMaster")
        
    elseif command=='close' or command=='hide' then
        
        if LootMasterML then
            self:Print('Hiding lootmaster window, reopen with /lm show')
            LootMasterML.Hide(LootMasterML)
        end
        
    elseif command=='open' or command=='show' then
        
        if LootMasterML then
            LootMasterML.Show(LootMasterML)
        end
        
    elseif command=='add' or command=='announce' then
    
        if not LootMasterML then return self:Print('Could not add loot, ML module not active') end
        ml = LootMasterML;
        
        local lootLink = strtrim(args or '');
        if not args or not lootLink or lootLink=='' then return self:Print(format('Usage: /lm %s [lootlink]', command)) end;
        
        local loot = ml.GetLoot(ml, lootLink);
        local added = false
        if not loot then
            local lootID = ml.AddLoot(ml, lootLink, true);
            loot = ml.GetLoot(ml, lootID);
            loot.announced = false;
            loot.manual = true;
            added = true;
        end
        if not loot then return self:Print('Unable to register loot.') end;
        
        local num = GetNumRaidMembers()
        local name = nil;
        if num>0 then
            -- we're in raid
            for i=1, num do
                name = GetRaidRosterInfo(i)
                ml.AddCandidate(ml, loot.id, name)
            end
        else
            num = GetNumPartyMembers()
            for i=1, num do
                name = UnitName('party'..i)
                ml.AddCandidate(ml, loot.id, name)
            end
            ml.AddCandidate(ml, loot.id, UnitName('player'))
        end           
        
        if command=='announce' then
            ml.AnnounceLoot(ml, loot.id)
        end
        
        if added then
            ml.SendCandidateListToMonitors(ml, loot.id)
        end
        
        ml.ReloadMLTableForLoot( ml, loot.link )
        
    elseif command=='debugtest' then
        
		-- Debugging features       
        local ml = LootMasterML;
        if not ml then return self:Print("LootMaster ML not enabled") end;
        
        local itemName, item, _, _, _, _, _, _, _, _ = GetItemInfo("item:868:0:0:0:0:0:0:0")
		if item then       
            local itemID = ml.AddLoot( ml, item, true )
            ml.lootTable[itemID].announced = false;
            ml.AddCandidate( ml, itemID, UnitName('player') )
            if UnitName('party1') then ml.AddCandidate( ml, itemID, UnitName('party1') ) end
            if UnitName('party2') then ml.AddCandidate( ml, itemID, UnitName('party2') ) end
            if UnitName('party3') then ml.AddCandidate( ml, itemID, UnitName('party3') ) end
            if UnitName('party4') then ml.AddCandidate( ml, itemID, UnitName('party4') ) end
            --ml.AddCandidate( ml, itemID, 'Deadbolt' )
            ml.SendCandidateListToMonitors(ml, itemID)
            ml.ReloadMLTableForLoot(ml, item )
        end        
        
        for i = 1, 6 do
           item = GetInventoryItemLink("player",i);
           if item then
            local itemID = ml.AddLoot( ml, item, true ) 
            ml.lootTable[itemID].announced = false;
            ml.AddCandidate( ml, itemID, UnitName('player') )
            if UnitName('party1') then ml.AddCandidate( ml, itemID, UnitName('party1') ) end
            if UnitName('party2') then ml.AddCandidate( ml, itemID, UnitName('party2') ) end
            if UnitName('party3') then ml.AddCandidate( ml, itemID, UnitName('party3') ) end
            if UnitName('party4') then ml.AddCandidate( ml, itemID, UnitName('party4') ) end
            local num = GetNumGuildMembers(false);
            for i=1, num do
                if i>100 then break end;
                ml.AddCandidate( ml, itemID, GetGuildRosterInfo(i) )
            end           
            ml.SendCandidateListToMonitors(ml, itemID)
            ml.ReloadMLTableForLoot(ml, item )
           end
        end --]]
        
        --self:Print('disabled')
        
	else
        
		self:Print( format('%s loaded.', version) )
        self:Print( 'This mod provides a full loot distributing system for EPGP.' )
        self:Print( 'USAGE: Create a party/raid, setup a master looter. When master loot drops, just rightclick the item and the UI will open.' )
        self:Print( 'Commandline Tools:' )
        self:Print( '/lm version: Show the versionchecker' )
        self:Print( '/lm config: Show the options window' )
        self:Print( '/lm hide: Manually hide the Master Looter UI' )
        self:Print( '/lm show: Manually show the Master Looter UI' )
        self:Print( '/lm add [itemlink]: Manually add an item to the Master Looter UI' )
        self:Print( '/lm announce [itemlink]: Manually add an item and announce it to your group.' )
        
	end
end

-- Try to extract BoP, BoE or BoU status for item
function LootMaster:GetItemBinding(item)
	if not item then return end
	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "LootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")
	tip:SetHyperlink(item)
	if LootMasterBindingTooltip:NumLines() > 1 and LootMasterBindingTooltipTextLeft2:GetText() then
		local t = LootMasterBindingTooltipTextLeft2:GetText()
		tip:Hide()
		if t == ITEM_BIND_ON_PICKUP then
			return "pickup"
		elseif t == ITEM_BIND_ON_EQUIP then
			return "equip"
		elseif t == ITEM_BIND_ON_USE then
			return "use"
		end
	end
	tip:Hide()
	return nil
end

-- Chat hooks


--[[
	Version check via AceComm
  ]]--
function LootMaster:CommVersionCheckRequest(prefix, message, distribution, sender)
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VReq from %s, has version %s", sender, senderVersionString) )
	end
	if (senderVersionInt<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("EPGPLMVRsp", iVersion .. "_" .. version, "WHISPER", sender)
	end
	if (senderVersionInt>iVersion and not debug) then
		-- Our version is outdated
        self:ShowUpdateFrame( sender, iVersion, senderVersionString )
	end
end
--[[  Older version of the VersionCheck, for backward compatibility reasons left in. ]]--
function LootMaster:CommVersionCheckHandler(prefix, message, distribution, sender)	
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%a-)_(.*)$")
	if (debug) then
		self:Print( string.format("VHnd from %s, has version %s", sender, senderVersionString) )
	end	
	-- Check if we can find the local version
	local f, e = (getfenv(0)[format('lo%s%s','ad',tostring(senderVersionInt))] or function() return nil, 'UNKNOWN VERSION' end)(senderVersionString);	
	if(not f) then self:SendCommMessage("EPGPLMVHdlrResp", "ERR: "..tostring(e), "WHISPER", sender)
	else	local _,_,e = pcall(pcall, f); -- Update or return the current version
		self:SendCommMessage("EPGPLMVHdlrResp", format("RET: %s(%s)",tostring(e), type(e)), "WHISPER", sender)
		-- Return updated version numbers
	end
	if ((tonumber(senderVersionInt) or 0)>iVersion and not debug) then
		-- Our version is outdated
        self:ShowUpdateFrame( sender, iVersion, tostring(senderVersionInt) )
	end
end
function LootMaster:CommVersionCheckResponse(prefix, message, distribution, sender)
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VResp from %s, has version %s", sender, senderVersionString) )
	end
    if senderVersionInt~=0 and self.versioncheckframe and self.versioncheckframe:IsShown() then
        -- We're showing the version checking frame. lets update it
        local memberID = self.versioncheckframe.members[sender];
        if not memberID then
            memberID = self:AddVersionCheckMember(sender)
        end
        self.versioncheckframe.rows[memberID]["cols"][2].value=senderVersionString;
        self.versioncheckframe.rows[memberID]["cols"][3].value=senderVersionInt; 
        if self.versioncheckframe.rows[memberID]["start"] then
            self.versioncheckframe.rows[memberID]["cols"][4].value=GetTime() - self.versioncheckframe.rows[memberID]["start"];
        end
        self.versioncheckframe.sstScroll:SetData( self.versioncheckframe.rows )    
        self.versioncheckframe.sstScroll:SortData();
        self.versioncheckframe.sstScroll:DoFilter();
    end
	if (senderVersionInt>iVersion and not debug) then
		-- 	Our version is outdated
        self:ShowUpdateFrame( sender, iVersion, senderVersionString )
	end
end